// CoJak: The CJK Oracular Assistant
// Copyright (C) 2003 by Tal Liron
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

package gnu.cojak;

import java.util.*;
import java.io.*;

public class ChineseCharacterIterator implements Iterator
{
	//
	// Construction
	//
	
	public ChineseCharacterIterator( File file ) throws IOException
	{
		reader = new BufferedReader( new FileReader( file ), 1048576 ); // 2^20
		prepareNext();
	}
	
	//
	// Iterator
	//
	
	public boolean hasNext()
	{
		return hasNext;
	}
	
	public Object next()
	{
		try
		{
			return chineseCharacter;
		}
		finally
		{
			prepareNext();
		}
	}
	
	public void remove()
	{
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	// Private
	
	private boolean hasNext = true;
	private BufferedReader reader;
	private ChineseCharacter chineseCharacter;
	private String line = null;
	
	private void prepareNext()
	{
		try
		{
			chineseCharacter = new ChineseCharacter();
			String oldCode = null, code, property, value;
			String[] tokens;
			int hex, macIver, hailu, siyan, meixian, end;
			while( true )
			{
				if( line == null ) line = reader.readLine();
				if( line == null )
				{
					// EOF
					hasNext = false;
					return;
				}
				if( line.charAt( 0 ) == '#' )
				{
					// Comments
					line = null;
					continue;
				}
				
				tokens = line.split( "\t" );
				code = tokens[0];
				if( ( oldCode != null ) && !code.equals( oldCode ) )
				{
					// New code - end this character
					break;
				}
				oldCode = code;
				property = tokens[1];
				value = tokens[2];
				line = null;
				
				// Translate hex values into unicode
				for( hex = value.indexOf( "&#x" ); hex != -1; hex = value.indexOf( "&#x", hex + 8 ) )
				{
					value = new StringBuffer( value ).replace( hex, hex + 8, "" + unhex( value.substring( hex + 3, hex + 7 ) ) ).toString();
				}
				
				if( property.equals( "fR/S" ) )
				{
					tokens = value.split( "\\." );
					if( chineseCharacter.kangxiRadical == null )
					{
						chineseCharacter.kangxiRadical = tokens[0];
						chineseCharacter.kangxiIndex = tokens[1];
					}
					else // new version doesn't use fAltR/S
					{
						chineseCharacter.kangxiAltRadical = tokens[0];
						chineseCharacter.kangxiAltIndex = tokens[1];
					}
				}
				else if( property.equals( "fAltR/S" ) ) // old version (new version uses another fR/S)
				{
					tokens = value.split( "\\." );
					chineseCharacter.kangxiAltRadical = tokens[0];
					chineseCharacter.kangxiAltIndex = tokens[1];
				}
				else if( property.equals( "fTotalStrokes" ) )
				{
					chineseCharacter.strokes = Integer.parseInt( value );
				}
				else if( property.equals( "fCangjie" ) )
				{
					chineseCharacter.cangjie = value;
				}
				else if( property.equals( "fFourCorner" ) )
				{
					chineseCharacter.fourCorner = value;
				}
				else if( property.equals( "fMacIver" ) ) // old version (new version uses fHakka)
				{
					chineseCharacter.hakkaMacIver = value;
				}
				else if( property.equals( "fRey" ) ) // old version (new version uses fHakka)
				{
					chineseCharacter.hakkaRey = value;
				}
				else if( property.equals( "fHagfaPinyim" ) ) // old version (new version uses fHakka)
				{
					chineseCharacter.hakkaHagfaPinyim = value;
				}
				else if( property.equals( "fSiyan" ) ) // old version (new version uses fHakka)
				{
					chineseCharacter.hakkaSiyan = value;
				}
				else if( property.equals( "fHailu" ) ) // old version (new version uses fHakka)
				{
					chineseCharacter.hakkaHailu = value;
				}
				else if( property.equals( "fHakka" ) ) // new version
				{
					macIver = value.indexOf( "[MacIver]" );
					hailu = value.indexOf( "[Hailu]" );
					siyan = value.indexOf( "[Siyan]" );
					meixian = value.indexOf( "[Meixian]" );
					if( macIver != -1 )
					{
						macIver += "[MacIver]".length() + 1;
						end = value.indexOf( '[', macIver ) - 1;
						if( end < 0 ) end = value.length();
						if( macIver < end )
						chineseCharacter.hakkaMacIver = value.substring( macIver, end );
					}
					if( hailu != -1 )
					{
						hailu += "[Hailu]".length() + 1;
						end = value.indexOf( '[', hailu ) - 1;
						if( end < 0 ) end = value.length();
						if( hailu < end )
						chineseCharacter.hakkaHailu = value.substring( hailu, end );
					}
					if( siyan != -1 )
					{
						siyan += "[Siyan]".length() + 1;
						end = value.indexOf( '[', siyan ) - 1;
						if( end < 0 ) end = value.length();
						if( siyan < end )
						chineseCharacter.hakkaSiyan = value.substring( siyan, end );
					}
					if( meixian != -1 )
					{
						meixian += "[Meixian]".length() + 1;
						end = value.indexOf( '[', meixian ) - 1;
						if( end < 0 ) end = value.length();
						if( meixian < end )
						chineseCharacter.hakkaMeixian = value.substring( meixian, end );
					}
				}
				else if( property.equals( "fCantonese" ) )
				{
					chineseCharacter.cantoneseJyutpin = value;
				}
				else if( property.equals( "fHanyu" ) ) // old version (new version uses fMandarin)
				{
					chineseCharacter.mandarinPinyin = value;
				}
				else if( property.equals( "fMandarin" ) ) // new version
				{
					chineseCharacter.mandarinPinyin = value;
				}
				else if( property.equals( "fTongyong" ) )
				{
					chineseCharacter.mandarinTongyong = value;
				}
				else if( property.equals( "fEnglish" ) )
				{
					chineseCharacter.english = value;
				}
			}
			
			chineseCharacter.chineseHanzi = unhex( oldCode.substring( 2, 6 ) );
		}
		catch( IOException x )
		{
			hasNext = false;
		}
	}
	
	private static char unhex( String hex )
	{
		char digit;
		int decimal = 0;
		char r = 0;
		int length = hex.length();
		int power = 1;
		for( int i = 0; i < length; i++ )
		{
			digit = hex.charAt( length - i - 1 );
			if( ( digit >= '0' ) && ( digit <= '9' ) )
			{
				decimal = digit - '0';
			}
			else if( ( digit >= 'a' ) && ( digit <= 'f' ) )
			{
				decimal = digit - 'a' + 10;
			}
			else if( ( digit >= 'A' ) && ( digit <= 'F' ) )
			{
				decimal = digit - 'A' + 10;
			}
			r += decimal * power;
			power *= 16;
		}
		return r;
	}
}
