// CoJak: The CJK Oracular Assistant
// Copyright (C) 2003 by Tal Liron
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

package gnu.cojak;

import org.apache.lucene.document.*;
import org.apache.lucene.search.*;
import org.apache.lucene.queryParser.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.io.*;
import java.util.*;

public class CoJaK extends JFrame implements ActionListener
{
	//
	// Main
	//
	
	public static void main( String[] arguments ) throws Exception
	{
		new CoJaK();
	}
	
	//
	// Construction
	//
	
	public CoJaK()
	{
		super( "CoJaK - The CJK Oracular Assistant" );
		
		addWindowListener
		(
			new WindowAdapter()
			{
				public void windowClosing( WindowEvent e )
				{
					System.exit( 0 );
				}
			}
		);
		addComponentListener
		(
			new ComponentAdapter()
			{
				public void componentResized( ComponentEvent e )
				{
					updateResults();
				}
			}
		);
		
		Properties config = new Properties();
		try
		{
			FileInputStream in = new FileInputStream( "cojak-config.txt" );
			config.load( in );
			in.close();
		}
		catch( Exception x )
		{
			config.setProperty( "font.character", "MingLiU" );
			config.setProperty( "font.kana", "MingLiU" );
			config.setProperty( "font.korean", "Batang" );
		}
		
		characterFont = new Font( config.getProperty( "font.character" ), Font.PLAIN, 26 );
		mixedFont = new Font( config.getProperty( "font.character" ), Font.PLAIN, 14 );
		japaneseFont = new Font( config.getProperty( "font.kana" ), Font.PLAIN, 14 );
		koreanFont = new Font( config.getProperty( "font.korean" ), Font.PLAIN, 14 );
		
		// Search panel
		JPanel grid = new JPanel();
		grid.setLayout( new GridLayout( 4, 2 ) );
		grid.add( asianType = new JComboBox() );
		grid.add( asian = new JTextField( 20 ) );
		grid.add( romanizationType = new JComboBox() );
		grid.add( romanization = new JTextField( 20 ) );
		grid.add( languageType = new JComboBox() );
		grid.add( language = new JTextField( 20 ) );
		grid.add( indexType = new JComboBox() );
		grid.add( index = new JTextField( 20 ) );
		JPanel buttons = new JPanel();
		buttons.setLayout( new GridLayout( 4, 1 ) );
		buttons.add( search = new JButton( "Search" ) );
		buttons.add( clear = new JButton( "Clear" ) );
		buttons.add( prev = new JButton( "Previous" ) );
		buttons.add( help = new JButton( "?" ) );
		JPanel searchPanel = new JPanel();
		searchPanel.add( grid );
		searchPanel.add( buttons );
		
		asianType.addItem( "Chinese:" );
		asianType.addItem( "Japanese:" );
		asianType.addItem( "Korean:" );
		asianType.addItem( "Chinese/Japanese/Korean:" );
		asianType.addActionListener( this );
		asian.setFont( characterFont );
		romanizationType.addActionListener( this );
		romanization.setFont( mixedFont );
		languageType.addItem( "English:" );
		search.setMnemonic( KeyEvent.VK_S );
		search.addActionListener( this );
		getRootPane().setDefaultButton( search );
		clear.setMnemonic( KeyEvent.VK_C );
		clear.addActionListener( this );
		prev.setMnemonic( KeyEvent.VK_P );
		prev.addActionListener( this );
		help.addActionListener( this );
		
		// Results panel
		results = new JTable();
		results.setRowHeight( characterFont.getSize() + 2 );
		results.setRowSelectionAllowed( false );
		results.setIntercellSpacing( new Dimension( 1, 2 ) );
		scroll = new JScrollPane( results );
		
		updateAsianType();
		
		getContentPane().setLayout( new BorderLayout() );
		getContentPane().add( searchPanel, BorderLayout.NORTH );
		getContentPane().add( scroll, BorderLayout.CENTER );
		
		// Center
		pack();
		center( this );
		
		show();
	}
	
	//
	// ActionListener
	//
	
	public void actionPerformed( ActionEvent e )
	{
		if( e.getSource() == search )
		{
			search();
		}
		else if( e.getSource() == clear )
		{
			clear();
		}
		else if( e.getSource() == prev )
		{
			previous();
		}
		else if( e.getSource() == help )
		{
			help();
		}
		else if( e.getSource() == asianType )
		{
			updateAsianType();
		}
		else if( e.getSource() == romanizationType )
		{
			updateRomanizationType();
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	// Private
	
	protected java.util.List history = new LinkedList();
	protected String query;
	protected java.util.List fields;
	
	protected Font characterFont;
	protected Font mixedFont;
	protected Font japaneseFont;
	protected Font koreanFont;
	
	protected JComboBox asianType;
	protected JTextField asian;
	protected JComboBox romanizationType;
	protected JTextField romanization;
	protected JComboBox languageType;
	protected JTextField language;
	protected JComboBox indexType;
	protected JTextField index;
	protected JButton search;
	protected JButton clear;
	protected JButton prev;
	protected JButton help;
	
	protected JScrollPane scroll;
	protected JTable results;
	protected java.util.List selectedColumns = new LinkedList();
	protected Hits hits;
	
	protected void search()
	{
		search(
			asianType.getSelectedItem(),
			asian.getText(),
			romanizationType.getSelectedItem(),
			romanization.getText(),
			indexType.getSelectedItem(),
			index.getText(),
			language.getText()
		);
	}
	
	protected void clear()
	{
		asian.setText( "" );
		romanization.setText( "" );
		language.setText( "" );
		index.setText( "" );
	}
	
	protected void previous()
	{
		if( history.size() > 1 )
		{
			Object[] entry = (Object[]) history.remove( history.size() - 1 );
			entry = (Object[]) history.remove( history.size() - 1 );
			asianType.setSelectedItem( (String) entry[ 0 ] );
			asian.setText( (String) entry[ 1 ] );
			romanizationType.setSelectedItem( entry[ 2 ] );
			romanization.setText( (String) entry[ 3 ] );
			indexType.setSelectedItem( entry[ 4 ] );
			index.setText( (String) entry[ 5 ] );
			language.setText( (String) entry[ 6 ] );
			search( entry[ 0 ], (String) entry[ 1 ], entry[ 2 ], (String) entry[ 3 ], entry[ 4 ], (String) entry[ 5 ], (String) entry[ 6 ] );
		}
	}
	
	protected void help()
	{
		JDialog dialog = new JDialog( this, "About CoJaK", true );
		try
		{
			JTextPane pane = new JTextPane();
			pane.setPage( new File( "help.html" ).toURL() );
			dialog.getContentPane().add( new JScrollPane( pane ) );
			fullscreen( dialog, 50 );
			dialog.show();
		}
		catch( Exception x )
		{
		}
	}
	
	protected void updateAsianType()
	{
		asianType.removeActionListener( this );
		romanizationType.removeActionListener( this );
		
		String type = (String) asianType.getSelectedItem();
		if( type.equals( "Chinese/Japanese/Korean:" ) )
		{
			romanizationType.removeAllItems();
			romanizationType.addItem( "Default Alphabets:" );
			romanizationType.addItem( "Mandarin Pinyin:" );
			romanizationType.addItem( "Mandarin Tongyong:" );
			romanizationType.addItem( "Cantonese Jyutpin:" );
			romanizationType.addItem( "Hakka MacIver:" );
			romanizationType.addItem( "Hakka Rey:" );
			romanizationType.addItem( "Hakka Hagfa Pinyim:" );
			romanizationType.addItem( "Hakka Siyan:" );
			romanizationType.addItem( "Hakka Hailu:" );
			romanizationType.addItem( "Hakka Meixian:" );
			romanizationType.addItem( "Japanese Kana:" );
			romanizationType.addItem( "Korean Hangul:" );
			romanizationType.addItem( "Korean Romanization:" );
			romanizationType.setSelectedItem( "Default Alphabets:" );
			indexType.removeAllItems();
			indexType.addItem( "Strokes:" );
			indexType.addItem( "Kangxi:" );
			indexType.addItem( "Kangxi Radical:" );
		}
		else if( type.equals( "Chinese:" ) )
		{
			romanizationType.removeAllItems();
			romanizationType.addItem( "Mandarin Pinyin:" );
			romanizationType.addItem( "Mandarin Tongyong:" );
			romanizationType.addItem( "Cantonese Jyutpin:" );
			romanizationType.addItem( "Hakka MacIver:" );
			romanizationType.addItem( "Hakka Rey:" );
			romanizationType.addItem( "Hakka Hagfa Pinyim:" );
			romanizationType.addItem( "Hakka Siyan:" );
			romanizationType.addItem( "Hakka Hailu:" );
			romanizationType.addItem( "Hakka Meixian:" );
			romanizationType.setSelectedItem( "Mandarin Pinyin:" );
			indexType.removeAllItems();
			indexType.addItem( "Strokes:" );
			indexType.addItem( "Kangxi:" );
			indexType.addItem( "Kangxi Radical:" );
		}
		else if( type.equals( "Japanese:" ) )
		{
			romanizationType.removeAllItems();
			romanizationType.addItem( "Japanese Kana:" );
			romanizationType.setSelectedItem( "Japanese Kana:" );
			indexType.removeAllItems();
			indexType.addItem( "Strokes:" );
		}
		else if( type.equals( "Korean:" ) )
		{
			romanizationType.removeAllItems();
			romanizationType.addItem( "Korean Hangul:" );
			romanizationType.addItem( "Korean Romanization:" );
			romanizationType.setSelectedItem( "Korean Hangul:" );
			indexType.removeAllItems();
		}
		
		asianType.addActionListener( this );
		romanizationType.addActionListener( this );
		
		updateRomanizationType();
	}
	
	protected void updateRomanizationType()
	{
		String type = (String) romanizationType.getSelectedItem();
		if( type == null ) return;
		type = type.substring( 0, type.length() - 1 );
		if( type.startsWith( "Mandarin " ) )
		{
			type = type.substring( "Mandarin ".length() );
		}
		else if( type.startsWith( "Cantonese " ) )
		{
			type = type.substring( "Cantonese ".length() );
		}
		else if( type.startsWith( "Hakka " ) )
		{
			type = type.substring( "Hakka ".length() );
		}
		else if( type.startsWith( "Japanese " ) )
		{
			type = type.substring( "Japanese ".length() );
		}
		else if( type.startsWith( "Korean " ) )
		{
			type = type.substring( "Korean ".length() );
		}
		
		selectedColumns.clear();
		addAsianColumns();
		selectedColumns.add( type );
		selectedColumns.add( "English" );
		if( !type.equals( "Korean:" ) ) selectedColumns.add( "Info" );
		search();
	}
	
	protected void addAsianColumns()
	{
		String type = (String) asianType.getSelectedItem();
		if( type.equals( "Chinese/Japanese/Korean:" ) )
		{
			selectedColumns.add( "Characters" );
		}
		else if( type.equals( "Chinese:" ) )
		{
			selectedColumns.add( "Hanzi" );
		}
		else if( type.equals( "Japanese:" ) )
		{
			selectedColumns.add( "Kanji" );
		}
		else if( type.equals( "Korean:" ) )
		{
			selectedColumns.add( "Hanja" );
		}
	}
	
	protected void search( Object asianType, String asian, Object romanizationType, String romanization, Object indexType, String index, String english )
	{
		query = "";
		fields = new LinkedList();
		addAsianSearchTerm( (String) asianType, asian );
		addRomanizationSearchTerm( (String) romanizationType, romanization );
		addIndexSearchTerm( (String) indexType, index );
		addSearchTerm( "english", english );
		
		if( !fields.isEmpty() )
		{
			try
			{
				Index searchIndex = new Index();
				hits = searchIndex.search( query, fields );
				updateResults();
				history.add( new Object[] {
					asianType,
					asian,
					romanizationType,
					romanization,
					indexType,
					index,
					english
				} );
			}
			catch( IOException x )
			{
			}
			catch( ParseException x )
			{
			}
		}
	}
	
	protected void addSearchTerm( String field, String value )
	{
		if( value.length() != 0 )
		{
			if( ( value.charAt( 0 ) == '\"' ) || ( value.indexOf( ' ' ) == -1 ) )
			{
				if( query.length() > 0 ) query += " AND ";
				query += field + ":" + value;
				fields.add( "field" );
			}
			else
			{
				String[] values = value.split( " " );
				for( int i = 0; i < values.length; i++ )
				{
					addSearchTerm( field, values[i] );
				}
			}
		}
	}
	
	protected void addAsianSearchTerm( String type, String value )
	{
		if( type.equals( "Chinese/Japanese/Korean:" ) )
		{
			if( value.length() != 0 )
			{
				fields.add( "chineseHanzi" );
				fields.add( "japaneseKanji" );
				fields.add( "koreanHanja" );
				query += "(chineseHanzi:" + value + " OR japaneseKanji:" + value + " OR koreanHanja:" + value + ")";
			}
		}
		else if( type.equals( "Chinese:" ) )
		{
			addSearchTerm( "chineseHanzi", value );
		}
		else if( type.equals( "Japanese:" ) )
		{
			addSearchTerm( "japaneseKanji", value );
		}
		else if( type.equals( "Korean:" ) )
		{
			addSearchTerm( "koreanHanja", value );
		}
	}
	
	protected void addRomanizationSearchTerm( String type, String value )
	{
		if( type.equals( "Mandarin Pinyin:" ) )
		{
			addSearchTerm( "mandarinPinyin", value );
		}
		else if( type.equals( "Mandarin Tongyong:" ) )
		{
			addSearchTerm( "mandarinTongyong", value );
		}
		else if( type.equals( "Cantonese Jyutpin:" ) )
		{
			addSearchTerm( "cantoneseJyutpin", value );
		}
		else if( type.equals( "Hakka MacIver:" ) )
		{
			addSearchTerm( "hakkaMacIver", value );
		}
		else if( type.equals( "Hakka Rey:" ) )
		{
			addSearchTerm( "hakkaRey", value );
		}
		else if( type.equals( "Hakka Hagfa Pinyim:" ) )
		{
			addSearchTerm( "hakkaHagfaPinyim", value );
		}
		else if( type.equals( "Hakka Siyan:" ) )
		{
			addSearchTerm( "hakkaSiyan", value );
		}
		else if( type.equals( "Hakka Hailu:" ) )
		{
			addSearchTerm( "hakkaHailu", value );
		}
		else if( type.equals( "Hakka Meixian:" ) )
		{
			addSearchTerm( "hakkaMeixian", value );
		}
		else if( type.equals( "Japanese Kana:" ) )
		{
			addSearchTerm( "japaneseKana", value );
		}
		else if( type.equals( "Korean Hangul:" ) )
		{
			addSearchTerm( "koreanHangul", value );
		}
		else if( type.equals( "Korean Romanization:" ) )
		{
			addSearchTerm( "koreanRomanization", value );
		}
	}
	
	protected void addIndexSearchTerm( String type, String value )
	{
		if( type == null ) return;
		if( type.equals( "Strokes:" ) )
		{
			addSearchTerm( "strokes", value );
		}
		else if( type.equals( "Kangxi:" ) )
		{
			addSearchTerm( "kangxi", value );
		}
		else if( type.equals( "Kangxi Radical:" ) )
		{
			addSearchTerm( "kangxiRadical", value );
		}
	}
	
	protected void updateResults()
	{
		if( hits == null ) return;
		
		java.util.List selectedRows = new LinkedList();
		Document document;
		String type = (String) asianType.getSelectedItem();
		for( int i = 0; i < hits.length(); i++ )
		{
			try
			{
				document = hits.doc( i );
				if( type.equals( "Chinese:" ) )
				{
					if( document.get( "chineseHanzi" ) == null )
						continue;
				}
				else if( type.equals( "Japanese:" ) )
				{
					if( ( document.get( "japaneseKanji" ) == null ) && ( document.get( "japaneseKana" ) == null ) )
						continue;
				}
				else if( type.equals( "Korean:" ) )
				{
					if( ( document.get( "koreanHangul" ) == null ) && ( document.get( "koreanRomanization" ) == null ) )
						continue;
				}
				selectedRows.add( document );
			}
			catch( IOException x )
			{
			}
		}
		
		int columns = selectedColumns.size();
		int rows = selectedRows.size();
		
		Object[][] data = new Object[ rows ][ columns ];;
		Object[] headers = new Object[ columns ];;
		
		for( int i = 0; i < rows; i++ )
		{
			document = (Document) selectedRows.get( i );
			for( int j = 0; j < columns; j++ )
			{
				String c = (String) selectedColumns.get( j );
				if( c.equals( "Characters" ) )
				{
					data[ i ][ j ] = document.get( "chineseHanzi" );
					if( data[ i ][ j ] == null ) data[ i ][ j ] = document.get( "japaneseKanji" );
					if( data[ i ][ j ] == null ) data[ i ][ j ] = document.get( "koreanHanja" );
				}
				else if( c.equals( "Hanzi" ) ) data[ i ][ j ] = document.get( "chineseHanzi" );
				else if( c.equals( "Kanji" ) ) data[ i ][ j ] = document.get( "japaneseKanji" );
				else if( c.equals( "Hanja" ) )
				{
					data[ i ][ j ] = document.get( "koreanHanja" );
					if( data[ i ][ j ] == null ) data[ i ][ j ] = document.get( "japaneseKanji" );
				}
				else if( c.equals( "Default Alphabets" ) )
				{
					data[ i ][ j ] = document.get( "mandarinPinyin" );
					if( data[ i ][ j ] == null ) data[ i ][ j ] = document.get( "japaneseKana" );
					if( data[ i ][ j ] == null ) data[ i ][ j ] = document.get( "koreanHangul" );
				}
				else if( c.equals( "Pinyin" ) ) data[ i ][ j ] = document.get( "mandarinPinyin" );
				else if( c.equals( "Tongyong" ) ) data[ i ][ j ] = document.get( "mandarinTongyong" );
				else if( c.equals( "Jyutpin" ) ) data[ i ][ j ] = document.get( "cantoneseJyutpin" );
				else if( c.equals( "MacIver" ) ) data[ i ][ j ] = document.get( "hakkaMacIver" );
				else if( c.equals( "Rey" ) ) data[ i ][ j ] = document.get( "hakkaRey" );
				else if( c.equals( "Hagfa Pinyim" ) ) data[ i ][ j ] = document.get( "hakkaHagfaPinyim" );
				else if( c.equals( "Siyan" ) ) data[ i ][ j ] = document.get( "hakkaSiyan" );
				else if( c.equals( "Hailu" ) ) data[ i ][ j ] = document.get( "hakkaHailu" );
				else if( c.equals( "Meixian" ) ) data[ i ][ j ] = document.get( "hakkaMeixian" );
				else if( c.equals( "Kana" ) ) data[ i ][ j ] = document.get( "japaneseKana" );
				else if( c.equals( "Hangul" ) ) data[ i ][ j ] = document.get( "koreanHangul" );
				else if( c.equals( "Romanization" ) ) data[ i ][ j ] = document.get( "koreanRomanization" );
				else if( c.equals( "English" ) ) data[ i ][ j ] = document.get( "english" );
				else if( c.equals( "Info" ) )
				{
					String text = "";
					if( document.get( "strokes" ) != null )
					{
						if( text.length() > 0 ) text += ", ";
						text += document.get( "strokes" ) + " strokes";
					}
					if( document.get( "kangxi" ) != null )
					{
						if( text.length() > 0 ) text += ", ";
						text += "Kangxi " + document.get( "kangxi" );
					}
					if( document.get( "cangjie" ) != null )
					{
						if( text.length() > 0 ) text += ", ";
						text += "Cangjie " + document.get( "cangjie" );
					}
					if( document.get( "fourCorner" ) != null )
					{
						if( text.length() > 0 ) text += ", ";
						text += "Four-Corner " + document.get( "fourCorner" );
					}
					data[ i ][ j ] = text;
				}
			}
		}
		
		selectedColumns.toArray( headers );
		results.setModel( new Model( data, headers ) );
		
		for( int i = 0; i < results.getColumnCount(); i++ )
		{
			TableColumn c = results.getColumnModel().getColumn( i );
			String header = (String) c.getHeaderValue();
			if( header.equals( "Hanzi" ) || header.equals( "Kanji" ) || header.equals( "Hanja" ) || header.equals( "Characters" ) )
			{
				c.setCellRenderer( new AsianRenderer( characterFont ) );
				c.setCellEditor( new AsianEditor( characterFont ) );
			}
			else if( header.equals( "Kana" ) )
			{
				c.setCellRenderer( new AsianRenderer( japaneseFont ) );
			}
			else if( header.equals( "Hangul" ) )
			{
				c.setCellRenderer( new AsianRenderer( koreanFont ) );
			}
			else if( header.equals( "Default Alphabets" ) )
			{
				c.setCellRenderer( new AsianRenderer( koreanFont ) );
			}
			else if( header.equals( "English" ) )
			{
				c.setCellRenderer( new MixedRenderer( mixedFont ) );
				c.setCellEditor( new MixedEditor() );
			}
			else if( header.equals( "Info" ) )
			{
				c.setCellRenderer( new MixedRenderer( null ) );
			}
		}
	}
	
	class Model extends DefaultTableModel
	{
		public Model( Object[][] data, Object[] headers )
		{
			super( data, headers );
		}
		
		public boolean isCellEditable( int row, int column )
		{
			TableColumn c = results.getColumnModel().getColumn( column );
			String header = (String) c.getHeaderValue();
			return header.equals( "Hanzi" ) || header.equals( "Kanji" ) || header.equals( "Hanja" ) || header.equals( "Characters" ) || header.equals( "English" );
		}
	}
	
	class AsianRenderer extends DefaultTableCellRenderer
	{
		public AsianRenderer( Font font )
		{
			super();
			this.font = font;
		}
		
		public Component getTableCellRendererComponent( JTable table, Object value, boolean selected, boolean focused, int row, int column )
		{
			super.getTableCellRendererComponent( table, value, selected, focused, row, column );
			setAlignmentY( TOP_ALIGNMENT );
			setFont( font );
			//TableColumn c = results.getColumnModel().getColumn( column );
			//c.setPreferredWidth( max( getMinimumSize().width, c.getPreferredWidth() ) );
			return this;
		}
		
		private Font font;
	}
	
	class AsianEditor extends DefaultCellEditor implements CaretListener
	{
		public AsianEditor( Font font )
		{
			super( new JTextField() );
			((JTextField) editorComponent).addCaretListener( this );
			((JTextField) editorComponent).setFont( font );
		}
		
		public Component getTableCellEditorComponent( JTable table, Object value, boolean selected, int row, int column )
		{
			Component c = super.getTableCellEditorComponent( table, value, selected, row, column );
			return c;
		}
		
		public void caretUpdate( CaretEvent e )
		{
			if( e.getDot() != e.getMark() )
			{
				String selected = ((JTextField) e.getSource()).getSelectedText();
				clear();
				asian.setText( selected );
				search();
			}
		}
	}
	
	class MixedRenderer extends JTextArea implements TableCellRenderer
	{
		public MixedRenderer( Font font )
		{
			super();
			setLineWrap( true );
			setWrapStyleWord( true );
			if( font != null ) setFont( font );
		}
		
		public Component getTableCellRendererComponent( JTable table, Object value, boolean selected, boolean focused, int row, int column )
		{
			setText( (String) value );
			results.setRowHeight( row, max( results.getRowHeight( row ), getMinimumSize().height ) );
			return this;
		}
	}
	
	class MixedEditor extends DefaultCellEditor implements CaretListener
	{
		public MixedEditor()
		{
			super( new JTextField() );
			((JTextField) editorComponent).addCaretListener( this );
			((JTextField) editorComponent).setFont( mixedFont );
		}
		
		public Component getTableCellEditorComponent( JTable table, Object value, boolean selected, int row, int column )
		{
			Component c = super.getTableCellEditorComponent( table, value, selected, row, column );
			return c;
		}
		
		public void caretUpdate( CaretEvent e )
		{
			if( e.getDot() != e.getMark() )
			{
				String selected = ((JTextField) e.getSource()).getSelectedText();
				clear();
				asian.setText( selected );
				search();
			}
		}
	}
	
	protected static void center( Window window )
	{
		Dimension size = window.getSize();
		Rectangle containerBounds = new Rectangle( 0, 0, Toolkit.getDefaultToolkit().getScreenSize().width, Toolkit.getDefaultToolkit().getScreenSize().height );
		window.setBounds( containerBounds.x + ( containerBounds.width - size.width ) / 2, containerBounds.y + ( containerBounds.height - size.height ) / 2, size.width, size.height );
	}
	
	protected static void fullscreen( Window window, int margin )
	{
		Dimension size = window.getSize();
		Rectangle containerBounds = new Rectangle( margin, margin, Toolkit.getDefaultToolkit().getScreenSize().width - margin * 2, Toolkit.getDefaultToolkit().getScreenSize().height - margin * 2 );
		window.setBounds( containerBounds );
	}
	
	protected static int min( int a, int b )
	{
		if( a < b ) return a; else return b;
	}
	
	protected static int max( int a, int b )
	{
		if( a > b ) return a; else return b;
	}
}

