// CoJak: The CJK Oracular Assistant
// Copyright (C) 2003 by Tal Liron
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

package gnu.cojak;

import java.util.*;
import java.io.*;

public class JapaneseCharacterIterator implements Iterator
{
	//
	// Construction
	//
	
	public JapaneseCharacterIterator( File file ) throws IOException
	{
		reader = new BufferedReader( new InputStreamReader( new FileInputStream( file ), "EUC-JP" ), 1048576 ); // 2^20
		prepareNext();
	}
	
	//
	// Iterator
	//
	
	public boolean hasNext()
	{
		return hasNext;
	}
	
	public Object next()
	{
		try
		{
			return japaneseCharacter;
		}
		finally
		{
			prepareNext();
		}
	}
	
	public void remove()
	{
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	// Private
	
	private boolean hasNext = true;
	private BufferedReader reader;
	private JapaneseCharacter japaneseCharacter;
	
	private void prepareNext()
	{
		try
		{
			japaneseCharacter = new JapaneseCharacter();
			String line = "#";
			int strokes, korean, mandarinPinyin, fourCorner, english, kana, end;
			while( line.charAt( 0 ) == '#' )
			{
				line = reader.readLine();
				if( line == null )
				{
					// EOF
					hasNext = false;
					return;
				}
			}
			japaneseCharacter.japaneseKanji = line.charAt( 0 );
			strokes = line.indexOf( " S" );
			korean = line.indexOf( " W" );
			mandarinPinyin = line.indexOf( " Y" );
			fourCorner = line.indexOf( " Q" );
			english = line.indexOf( "{" );
			if( strokes != -1 )
			{
				end = line.indexOf( ' ', strokes + 3 );
				japaneseCharacter.strokes = Integer.parseInt( line.substring( strokes + 2, end ) );
			}
			if( korean != -1 )
			{
				end = line.indexOf( ' ', korean + 3 );
				japaneseCharacter.koreanRomanization = line.substring( korean + 2, end );
			}
			if( mandarinPinyin != -1 )
			{
				end = line.indexOf( ' ', mandarinPinyin + 3 );
				japaneseCharacter.mandarinPinyin = line.substring( mandarinPinyin + 2, end );
			}
			if( fourCorner != -1 )
			{
				end = line.indexOf( ' ', fourCorner + 3 );
				japaneseCharacter.fourCorner = line.substring( fourCorner + 2, end );
			}
			if( english != -1 )
			{
				end = line.length();
				japaneseCharacter.english = line.substring( english, end );
			}
			else
			{
				english = line.length();
			}
			for( kana = 2; kana < english; kana++ )
			{
				if( Character.getType( line.charAt( kana ) ) == Character.OTHER_LETTER )
				{
					japaneseCharacter.japaneseKana = line.substring( kana, english - 1 );
					break;
				}
			}
		}
		catch( IOException x )
		{
			hasNext = false;
		}
	}
}
