// CoJak: The CJK Oracular Assistant
// Copyright (C) 2003 by Tal Liron
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

package gnu.cojak;

import java.util.*;
import java.io.*;

public class JapaneseWordIterator implements Iterator
{
	//
	// Construction
	//
	
	public JapaneseWordIterator( File file ) throws IOException
	{
		reader = new BufferedReader( new InputStreamReader( new FileInputStream( file ), "EUC-JP" ), 1048576 ); // 2^20
		reader.readLine();
		prepareNext();
	}
	
	//
	// Iterator
	//
	
	public boolean hasNext()
	{
		return hasNext;
	}
	
	public Object next()
	{
		try
		{
			return japaneseWord;
		}
		finally
		{
			prepareNext();
		}
	}
	
	public void remove()
	{
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	// Private
	
	private boolean hasNext = true;
	private BufferedReader reader;
	private JapaneseWord japaneseWord;
	
	private void prepareNext()
	{
		try
		{
			japaneseWord = new JapaneseWord();
			int kanjiEnd, kanaStart, kanaEnd;
			String line = reader.readLine();
			if( line == null )
			{
				// EOF
				hasNext = false;
				return;
			}
			kanjiEnd = line.indexOf( ' ' );
			kanaStart = line.indexOf( '[', kanjiEnd + 1 );
			if( kanaStart == -1 )
			{
				japaneseWord.japaneseKanji = null;
				japaneseWord.japaneseKana = line.substring( 0, kanjiEnd );
				japaneseWord.english = line.substring( kanjiEnd + 1 );
			}
			else
			{
				kanaEnd = line.indexOf( ']', kanaStart + 1 );
				japaneseWord.japaneseKanji = line.substring( 0, kanjiEnd );
				japaneseWord.japaneseKana = line.substring( kanaStart + 1, kanaEnd );
				japaneseWord.english = line.substring( kanaEnd + 2 );
			}
		}
		catch( IOException x )
		{
			hasNext = false;
		}
	}
}
