// CoJak: The CJK Oracular Assistant
// Copyright (C) 2003 by Tal Liron
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

package gnu.cojak;

import java.util.*;
import java.io.*;

public class KoreanWordIterator implements Iterator
{
	//
	// Construction
	//
	
	public KoreanWordIterator( File file ) throws IOException
	{
		reader = new BufferedReader( new InputStreamReader( new FileInputStream( file ), "EUC-KR" ), 1048576 ); // 2^20
		prepareNext();
	}
	
	//
	// Iterator
	//
	
	public boolean hasNext()
	{
		return hasNext;
	}
	
	public Object next()
	{
		try
		{
			return koreanWord;
		}
		finally
		{
			prepareNext();
		}
	}
	
	public void remove()
	{
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	// Private
	
	private boolean hasNext = true;
	private BufferedReader reader;
	private KoreanWord koreanWord;
	
	private void prepareNext()
	{
		try
		{
			koreanWord = new KoreanWord();
			String line = "#";
			int wordEnd, pinyinStart, pinyinEnd;
			while( line.charAt( 0 ) == '#' )
			{
				line = reader.readLine();
				if( line == null )
				{
					// EOF
					hasNext = false;
					return;
				}
			}
			int nextTab;
			int num = 0;
			String token;
			for( int tab = line.indexOf( '\t' ); tab != line.length(); num++ )
			{
				nextTab = line.indexOf( '\t', tab + 1 );
				if( nextTab == -1 ) nextTab = line.length();
				token = line.substring( tab + 1, nextTab );
				tab = nextTab;
				
				if( num == 0 )
				{
					koreanWord.english = token;
				}
				else if( num == 1 )
				{
					if( token.length() > 0 ) koreanWord.english += " (" + token + ")";
				}
				else if( num == 3 )
				{
					if( token.length() > 0 ) koreanWord.koreanHangul = token;
				}
				else if( num == 4 )
				{
					if( token.length() > 0 ) koreanWord.koreanHanja = token;
				}
			}
		}
		catch( IOException x )
		{
			hasNext = false;
		}
	}
}
